"""Inserting initial data

Revision ID: ac95cbd0fce
Revises: 99fda072ff9
Create Date: 2015-09-05 20:06:19.814067

"""

# revision identifiers, used by Alembic.
revision = 'ac95cbd0fce'
down_revision = '99fda072ff9'

from alembic import op
import sqlalchemy as sa


UPGRADE = """
INSERT INTO usergroups (name, description) VALUES ('admins', '');
INSERT INTO usergroups (name, description) VALUES ('users', '');
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (1,'modules','ringo.model.modul.ModulItem','Modul','Modules','','%s|name','admin-menu',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (2,'actions','ringo.model.modul.ActionItem','Action','Actions','','%s (%s/%s)|name,modul,url','hidden',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (3,'users','ringo.model.user.User','User','Users','','%s|login','admin-menu',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (4,'usergroups','ringo.model.user.Usergroup','Usergroup','Usergroups','','%s|name','admin-menu',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (5,'roles','ringo.model.user.Role','Role','Roles','','%s|name','admin-menu',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (6,'profiles','ringo.model.user.Profile','Profile','Profiles','','%s %s|first_name,last_name','admin-menu',NULL);
INSERT INTO modules (id, name, clazzpath, label, label_plural, description, str_repr, display, default_gid) VALUES (14,'forms','ringo.model.form.Form','Form','Forms','','%s|title','admin-menu',NULL);
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (1, 1, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (2, 1, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (3, 1, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (4, 3, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (5, 3, 'Create', 'create', ' icon-plus', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (6, 3, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (7, 3, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (8, 3, 'Delete', 'delete/{id}', 'icon-trash', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (9, 4, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (10, 4, 'Create', 'create', ' icon-plus', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (11, 4, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (12, 4, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (13, 4, 'Delete', 'delete/{id}', 'icon-trash', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (14, 5, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (15, 5, 'Create', 'create', ' icon-plus', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (16, 5, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (17, 5, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (18, 5, 'Delete', 'delete/{id}', 'icon-trash', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (19, 6, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (20, 6, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (21, 6, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (22, 2, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (23, 2, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (24, 3, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (25, 3, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (26, 4, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (27, 4, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (28, 5, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (29, 5, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (30, 14, 'List', 'list', 'icon-list-alt', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (31, 14, 'Create', 'create', ' icon-plus', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (32, 14, 'Read', 'read/{id}', 'icon-eye-open', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (33, 14, 'Update', 'update/{id}', 'icon-edit', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (34, 14, 'Delete', 'delete/{id}', 'icon-trash', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (35, 14, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (36, 14, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (37, 6, 'Export', 'export/{id}', 'icon-export', '', true, 'primary', '');
INSERT INTO actions (id, mid, name, url, icon, description, bundle, display, permission) VALUES (38, 6, 'Import', 'import', 'icon-import', '', false, 'primary', '');
INSERT INTO roles (id, label, name, description, admin, gid, uid) VALUES (1, 'Users', 'user', '', false, NULL, NULL);
INSERT INTO roles (id, label, name, description, admin, gid, uid) VALUES (2, 'Administration', 'admin', '', true, NULL, NULL);
INSERT INTO nm_action_roles (aid, rid) VALUES (20, 1);
INSERT INTO nm_action_roles (aid, rid) VALUES (21, 1);
INSERT INTO user_settings (id, settings) VALUES (1, '{}');
INSERT INTO users (id, login, password, activated, activation_token, sid, last_login, default_gid, gid, uid) VALUES (1, 'admin', '5ebe2294ecd0e0f08eab7690d2a6ee69', true, '', 1, '2015-09-05 18:43:06.178858', NULL, NULL, NULL);
INSERT INTO profiles (id, first_name, last_name, gender, birthday, address, phone, email, web, uid, gid) VALUES (1, '', '', NULL, NULL, '', '', '', '', 1, NULL);
INSERT INTO nm_user_roles (uid, rid) VALUES (1, 2);
UPDATE roles SET uid = 1;
UPDATE roles SET gid = 1;
UPDATE usergroups SET uid = 1;
UPDATE usergroups SET gid = 1;
UPDATE users SET uid = 1;
UPDATE users SET gid = 1;
"""
DOWNGRADE = """
DELETE FROM profiles;
DELETE FROM users;
DELETE FROM nm_action_roles;
DELETE FROM actions;
DELETE FROM modules;
DELETE FROM usergroups;
"""


def iter_statements(stmts):
    for st in [x for x in stmts.split('\n') if x]:
        op.execute(st)


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###
    iter_statements(UPGRADE)


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###
    iter_statements(DOWNGRADE)



from ringo.model.base import BaseItem, BaseList
from ringo.lib.imexport import JSONExporter


class JSONResponse(object):
    """Generic response item for JSON responses on the RESTfull api"""

    def __init__(self, success, data, params=None):
        """Create a new JSONResponse. The Response object has the
        following format::

        {
            "success":[true|false],
            "data": object
            "params": {
                "parm1": "value1",
                "parm2": "value1",
                ...
                "parmN": "value1"
            }
        }

        The format of the data or params attribute is not specified. It
        can basically contain any valid JSON. Usually the data attribute
        will contain a dictionary with the attributes of an item or a
        list of  dictionarys with values of items.

        :data: Payload included in the Response.
        :success: Generic status of the query
        :params: Optional parameters. Can include further information.
        E.g why the last request failed.
        """
        self._data = data
        self._success = success
        self._params = params

    def __json__(self, request):
        rvalue = {}
        rvalue['success'] = self._success
        if isinstance(self._data, BaseItem):
            exporter = JSONExporter(self._data.__class__)
            self._data = exporter.perform([self._data])
        elif isinstance(self._data, BaseList):
            exporter = JSONExporter(self._data.__class__)
            self._data = exporter.perform(self._data.items)
        rvalue['data'] = self._data
        rvalue['params'] = self._params
        return rvalue



"""
Modules are the blocks in Ringos building block archticture!
The term 'Modul' decribes the collection of all needed infrastructure to work
and store certain type of data. Where certain type of
data means users, files, movies etc. rather than integers or date values.

The moduls infrastructure consists of

1. *Views* to handle incoming request and generating the proper responses.
2. *Templates* to define the layout of the pages of the module
3. *Model* which defines the data fields of your modul.
4. *Forms* and *Table* confiuration which defines how forms and overview
   pages will be rendered.

Ringo provides some default infrastructure which is used for all modules
as long the modul does not implements it in a custom way. This default
infrastructure is present for *Views* and *Templates*. For those the
default views and templates are used. The views provide default LCRUDIE
:ref:`.ActionItem` for all modules:

* List: Listing all items of the module. This is slightly different to
  the action to read a single item.
* Create: Create new items.
* Read: Show the item in detail in readonly mode.
* Update: Edit items of the module.
* Delete: Deleting items.
* Import (CSV, JSON)
* Export (CSV, JSON)

These actions are defined in the :attr:`.ACTIONS` modul varialble.

Ringo already comes with many predifend moduls. Please refer to
documentation to see which moduls are available.

"""

import logging
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.user import BaseItem
from ringo.lib.helpers import dynamic_import

log = logging.getLogger(__name__)


class ActionItem(BaseItem, Base):
    """A ActionItem is the configuration and representation of the
    :class:`.ModulItem` configured actions in the application. This
    class configures aspected of rendering and which view is called if
    the user selects the action."""
    __tablename__ = 'actions'
    _modul_id = 2
    id = sa.Column(sa.Integer, primary_key=True)
    mid = sa.Column(sa.Integer, sa.ForeignKey('modules.id'))
    """Foreign Key to the modull to which the action belongs to."""
    name = sa.Column(sa.String, nullable=False)
    """Name of the Action. E.g "Create" or "Read". The will be used for
     * rendering the action and
     * internal identification (lowercase version of the name) on
     * permission checks (see :attr:`.permission`).
    """
    url = sa.Column(sa.String, nullable=False)
    """The url which is called for this action if a user calls this
    actions. Used for rendering links in the application and while
    initialising the moduls on application start to setup the views."""
    icon = sa.Column(sa.String, nullable=False, default='')
    """A string which configures which icons should be used when
    rendering the action in the UI. The string will be usually rendered
    as class attribute of a bootstrap <i> element. Known predefined values are:

     * List: icon-list-alt
     * Add: icon-plus
     * Read: icon-eye-open
     * Edit: icon-edit
     * Delete: icon-eye-delete, icon-trash
     * Download: icon-download
     * Export: icon-export
     * Import: icon-import
     """
    description = sa.Column(sa.Text, nullable=False, default='')
    """Short description what the action does. Should be obvious by the
    name of the action anyway."""
    bundle = sa.Column(sa.Boolean, nullable=False, default=False)
    """Flag to indicate if the action should be available in the bundled
    actions"""
    display = sa.Column(sa.String, nullable=False, default="primary")
    """Optional. Configures a) If the action will be rendered at all and
    b) where to render the action in the context menu.

    The display variable is a comma separated list of values which
    defines the visibility. The following values are valid:

    ============== ===========
    value          description
    ============== ===========
    primary        Action is globally visible and will be rendered in
                   the primary context menu.
    secondary      Action is globally visible and will be rendered in
                   the secondary menu.
    hide           Completly hide the action. This effects both context
                   menu and overview.
    hide-context   Kind of modifier in addition to the `primary` and
                   `secondary` setting. Adding this value will still show
                   the action in the context but hide the action in the
                   overview.
    hide-overview  Kind of modifier in addition to the `primary` and
                   `secondary` setting. Adding this value will still show
                   the action in the overview but hide the action in the
                   context menu.
    ============== ===========

    Examples:

    * "primary,hide-context": Show action in overview only
    * "primary,hide-overview": Show action in context only, render
      action in primary context menu
    * "secondary": Show action globally, render action in secondary menu
      of context menu.
    * "hide": Completly hide the action.

    """
    permission = sa.Column(sa.String, nullable=False, default='')
    """Optional. Configure an alternative permission the user must have
    to be allowed to call this action. Known values are 'list', 'create',
    'read', 'update', 'delete', 'import', 'export'. If empty the
    permission system will use the the lowered name of the action."""

    def is_visible(self, location):
        """Checks based on the setting in the display configuration
        attribute if the action is visible in the given location.
        Location can be be `overview` or `context`.

        :location: String of location to check
        :returns: True or False
        """
        display = self.display.split(",")
        if "hide" in display:
            return False
        if location == "overview" and "hide-overview" in display:
            return False
        if location == "context" and "hide-context" in display:
            return False
        return True


class ModulItem(BaseItem, Base):
    """A ModulItem is the representation and configuration of a
    :mod:`.modul` in the application.

    Each module has a common set of configuration options which can be
    configured directly in the web interface and are stored in instances
    of :class:`.ModulItem`.  This includes the configuration of

    * visual aspects and how the modul and/or items of the modul will be
      rendered or displayed.
    * model configuration as which :class:`.ActionItem` should be
      available.

    Default usergroup: Write me!
    """

    __tablename__ = 'modules'
    _modul_id = 1
    _sql_eager_loads = ['actions']
    id = sa.Column(sa.Integer, primary_key=True)
    """Internal ID of the modul."""
    name = sa.Column(sa.String, unique=True, nullable=False)
    """Internal name of the modul which is usually autogenrated and must
    not be changed!. The name is crucial for the application as it also
    defines some other namings as the name of configuration files for
    forms or tables."""
    clazzpath = sa.Column(sa.String, unique=True, nullable=False)
    """Path to the moduls model in dot notation"""
    label = sa.Column(sa.String, nullable=False, default='')
    """The label in single form of the modul. Used for display in
    various places of the application."""
    label_plural = sa.Column(sa.String, nullable=False, default='')
    """The label in plural form of the modul. Used for display in
    various places of the application."""
    description = sa.Column(sa.Text, nullable=False, default='')
    """The label in plural form of the modul. Used for display in
    various places of the application."""
    str_repr = sa.Column(sa.String, nullable=False, default='%s|id')
    """Format string which defines how items of the modul will be renderered
    when the __unicode__ or __str__ method is called for the items. This
    is often the case in dropdown lists or overview tables. The string
    is a bar seperated string and defined as '%s-%s|foo, bar'. The left
    side of the configuration defines the "layout" with placeholder (%s)
    and the right part the attributes which will be used to replace the
    placeholders."""
    display = sa.Column(sa.String, nullable=False, default='hidden')
    """Configures where the items will be displayed in the web
    application. Possibile options are:

     * header-menu
     * user-menu
     * admin-menu
     * hidden
    """
    default_gid = sa.Column(sa.Integer, sa.ForeignKey('usergroups.id'))
    """Link to a usergroup which will be set as default useroup for new
    items of the modul if nothing other is defined."""

    default_group = sa.orm.relationship("Usergroup", uselist=False,
                                        foreign_keys=[default_gid])
    actions = sa.orm.relationship("ActionItem",
                                  backref="modul",
                                  lazy="joined",
                                  cascade="all")
    """List of :class:`.ActionItem` which are available for the modul."""

    _sql_eager_loads = ['actions.roles']
    """Preload the actions and associated roles to the action of the
    modul.  Are needed for permission checks. This will reduce the
    number of SQL-queries very much!"""

    @property
    def clazzbases(self):
        """Property which returns the base classes of the model of the
        modul. This is usefull and currently used for filtering in
        forms. e.g filtering modules which are tagable"""
        return [b.__name__ for b in self.get_clazz().__bases__]

    def get_clazz(self):
        """Returns the class defined in the clazzpath attribute.

        :returns: :class:`.BaseItem`
        """
        return dynamic_import(self.clazzpath)

    def get_label(self, plural=False):
        """Returns the label of the modul.

        :plural: If true the plural form is returned.
        :returns: String with label"""
        if plural:
            return self.label_plural
        return self.label

    def has_action(self, name):
        """Will return True if the modul has a ActionItem configured
        with given name. Else false.

        :name: Name of the action 
        :returns: True or False
        """
        for action in self.actions:
            if action.name.lower() == name.lower():
                return True
        return False


    def get_action(self, name):
        """Will return action item if the modul has a ActionItem configured
        with given name. Else None.

        :name: Name of the action
        :returns: ActionItem or None
        """
        for action in self.actions:
            if action.name.lower() == name.lower():
                return action
        return None


    def get_str_repr(self):
        """Return a tupel with format str and a list of fields."""
        # "%s - %s"|field1, field2 -> ("%s - %s", ["field1", "field2"])
        try:
            format_str, fields = self.str_repr.split("|")
            return (format_str, [f.strip() for f in fields.split(",")])
        except:
            return ("%s", ["id"])

_ = lambda msgid: msgid
ACTIONS = {
    "list":   ActionItem(name=_("List"),
                         url="list",
                         icon="icon-list-alt"),
    "create": ActionItem(name=_("Create"),
                         url="create",
                         icon=" icon-plus"),
    "read":   ActionItem(name=_("Read"),
                         url="read/{id}",
                         icon="icon-eye-open"),
    "update": ActionItem(name=_("Update"),
                         url="update/{id}",
                         icon="icon-edit"),
    "delete": ActionItem(name=_("Delete"),
                         url="delete/{id}",
                         icon="icon-trash",
                         bundle=True),
    "import": ActionItem(name=_("Import"),
                         url="import",
                         icon="icon-import"),
    "export": ActionItem(name=_("Export"),
                         url="export/{id}",
                         icon="icon-export",
                         bundle=True)
}
"""Default available actions in Ringo"""



import os
import md5
from dogpile.cache.region import make_region

# Cache initialisation
########################

# dogpile cache regions.
regions = {}


def init_cache(cachedir, regions):

    if regions is None:
        regions = []

    if not os.path.exists(cachedir):
        #raw_input("Will create datafiles in %r.\n"
        #            "To reset the cache + database, delete this
        #            directory.\n" "Press enter to continue.\n" % root)
        os.makedirs(cachedir)

    for region in regions:
        name = region[0]
        time = int(region[1])
        create_region(cachedir, name, time)

    # optional; call invalidate() on the region
    # once created so that all data is fresh when
    # the app is restarted.  Good for development,
    # on a production system needs to be used carefully
    invalidate_cache()


def md5_key_mangler(key):
    """Receive cache keys as long concatenated strings;
    distill them into an md5 hash.

    """
    return md5.md5(key).hexdigest()


def create_region(cachedir, name, time):
    regions[name] = make_region(
                # the "dbm" backend needs
                # string-encoded keys
                key_mangler=md5_key_mangler
            ).configure(
            # using type 'file' to illustrate
            # serialized persistence.  Normally
            # memcached or similar is a better choice
            # for caching.
            'dogpile.cache.dbm',
            expiration_time=time,
            arguments={
                "filename": os.path.join(cachedir, "cache.dbm")
            }
        )


def invalidate_cache(cache_regions=[]):
    for key in regions.keys():
        if len(cache_regions) == 0 or key in cache_regions:
            regions[key].invalidate()





from ringo.views.base.create import (
    create,
    rest_create
)
from ringo.views.base.list_ import (
    list_,
    bundle_,
    rest_list
)
from ringo.views.base.read import (
    read,
    rest_read
)
from ringo.views.base.update import (
    update,
    rest_update
)
from ringo.views.base.delete import (
    delete,
    rest_delete,
)
from ringo.views.base.export import (
    export,
)
from ringo.views.base.import_ import (
    import_,
)
from ringo.views.base.ownership import (
    ownership,
)

web_action_view_mapping = {
    "default": {
        "list": list_,
        "create": create,
        "read": read,
        "update": update,
        "delete": delete,
        "import": import_,
        "export": export,
        "bundle": bundle_,
        "ownership": ownership
    }
}

rest_action_view_mapping = {
    "default": {
        "list": rest_list,
        "create": rest_create,
        "read": rest_read,
        "update": rest_update,
        "delete": rest_delete,
    }
}

def get_action_view(mapping, action, module):
    if module in mapping:
        views = mapping[module]
        if action in views:
            return views[action]
    return mapping["default"].get(action)


def set_web_action_view(key, view, module="default"):
    if module in web_action_view_mapping:
        mod_actions = web_action_view_mapping.get(module)
    else:
        web_action_view_mapping[module] = {}
        mod_actions = web_action_view_mapping.get(module)
    mod_actions[key] = view
    web_action_view_mapping[module] = mod_actions


def set_rest_action_view(key, view, module="default"):
    if module in rest_action_view_mapping:
        mod_actions = rest_action_view_mapping.get(module)
    else:
        rest_action_view_mapping[module] = {}
        mod_actions = rest_action_view_mapping.get(module)
    mod_actions[key] = view
    rest_action_view_mapping[module] = mod_actions



import pkg_resources

from pyramid.view import view_config

from ringo.lib.helpers import (
    get_ringo_version,
    get_app_version,
    get_app_name,
    get_app_title
)
from ringo.lib.renderer import (
    DTListRenderer
)

from ringo.views.request import handle_history


@view_config(route_name='home', renderer='/index.mako')
def index_view(request):
    handle_history(request)
    values = {}
    return values


@view_config(route_name='about', renderer='/about.mako')
def about_view(request):
    handle_history(request)
    values = {}
    values['app_title'] = get_app_title()
    return values


@view_config(route_name='contact', renderer='/contact.mako')
def contact_view(request):
    handle_history(request)
    return {}


@view_config(route_name='version', renderer='/version.mako')
def version_view(request):
    # Fetch the versions of some Packages
    # Ringo
    handle_history(request)
    values = {}
    formbar_pkg = pkg_resources.get_distribution('formbar')
    sqla_pkg = pkg_resources.get_distribution('sqlalchemy')
    pyramid_pkg = pkg_resources.get_distribution('pyramid')
    values['app_title'] = get_app_title()
    values['app_version'] = get_app_version()
    values['app_name'] = get_app_name()
    values['ringo_version'] = get_ringo_version()
    values['app_version'] = get_app_version()
    values['formbar_version'] = formbar_pkg.version
    values['sqlalchemy_version'] = sqla_pkg.version
    values['pyramid_version'] = pyramid_pkg.version
    return values



import logging
from formbar.form import Form
from ringo.lib.form import get_form_config
from ringo.views.response import JSONResponse
from ringo.views.helpers import (
    get_item_form,
    render_item_form,
)
from ringo.views.request import (
    handle_params,
    handle_history,
    handle_POST_request,
    handle_redirect_on_success,
    get_return_value
)

log = logging.getLogger(__name__)


def create(request, callback=None, renderers=None,
           validators=None, values=None):
    """Base method to handle create requests. This view will render a
    create form to update items on (GET) requests.

    It the user submits the data (POST) that the data will be validated.
    If validation is successfull the item will be saved in the datebase
    and the callback method will be called. Finally a redirect to either
    the backurl or the edit or read mode of the item will be triggered.

    If the validation fails, the form will be rendered again.

    :request: Current request
    :callback: Current function which is called after the item has been read.
    :renderers: Dictionary of external renderers which should be used
                for renderering some form elements.
    :validators: List of external formbar validators which should be
                 added to the form for validation
    :values: Dictionary of additional values which will be available in
             the form
    :returns: Dictionary or Redirect.
    """
    handle_history(request)
    params = handle_params(request)

    # Create a new item
    clazz = request.context.__model__
    try:
        factory = clazz.get_item_factory(request)
    except TypeError:
        # Old version of get_item_factory method which does not take an
        # request parameter.
        factory = clazz.get_item_factory()
        factory._request = request
    # Only create a new item if there isn't already an item in the
    # request.context. This can happen if we define a custom view for
    # create where the item gets created before to set some default
    # values e.g (See create function of forms)
    if not request.context.item:
        request.context.item = factory.create(request.user, {})
    if values is None:
        values = {}
    values['_roles'] = [str(r.name) for r in request.user.roles]
    values.update(params.get('values', {}))
    form_values = request.session.get("form_values") or {}
    values.update(form_values)
    request.session["form_values"] = None
    request.session.save()

    form = get_item_form(params.get("form", "create"),
                         request, renderers, validators, values=values)
    if request.POST and 'blobforms' not in request.params:
        if handle_POST_request(form, request, callback, 'create', renderers):
            return handle_redirect_on_success(request)
    rvalues = get_return_value(request)
    rvalues['form'] = render_item_form(request, form, values, False)
    return rvalues


def rest_create(request, callback=None):
    """Create a new item of type clazz. The item will be
    initialised with the data provided in the submitted POST request.
    The submitted data will be validated before the item is actually
    saved. If the submission fails the item is not saved in the
    database. In all cases the item is returned as JSON object with the
    item and updated values back to the client. The JSON Response will
    include further details on the reason why the validation failed.

    :clazz: Class of item to create
    :request: Current request
    :returns: JSON object.

    """
    clazz = request.context.__model__
    # Create a new item.
    factory = clazz.get_item_factory()
    item = factory.create(request.user)
    # Initialise the create form for the item to be able to validate the
    # submitted data.
    form = Form(get_form_config(item, 'create'),
                item, request.db, translate=request.translate,
                csrf_token=request.session.get_csrf_token())
    if form.validate(request.params):
            sitem = form.save()
            return JSONResponse(True, sitem)
    else:
        # Validation fails! return item
        return JSONResponse(False, sitem)



"""Caching of items."""
import logging
import datetime
from pyramid.events import NewRequest

log = logging.getLogger(__name__)


class Cache(object):

    """Cache container to store elements"""

    def __init__(self, validity_period=0):
        """Intitialises a new Cache container. You can set the the
        validity_period in seconds. This parameter will ensure that the
        cache will stay available at least the amount of seconds defined
        in validity_period even if the clear method of the cache has
        been called earlier. If the no validity_period is provided. The
        cache will be cleared on every call of the clear method.

        :validity_period: Number of seconds the cache will be available,
        before it can be cleared.

        """
        self._created = datetime.datetime.now()
        self._validity_period = validity_period
        self._data = {}
        log.debug("New cache created")

    def clear(self, force=False):
        """Will clear cache if it is older then the the validity_period
        of the cache. If force is true the cache will be cleared even if
        the validity_period has not been exeeded.

        :force: Force clearing the cache. Defaults to False
        :returns: None

        """
        td = (datetime.datetime.now() - self._created)
        if force or td.seconds > self._validity_period:
            self._data = {}

    def set(self, key, value):
        """Will set a new value for the given key in the cache. If there
        is already a value stored then the value will be overwritten.

        :key: String idenditifier for the cached value
        :value: The value to cache
        :returns: None

        """
        self._data[key] = value

    def get(self, key):
        """Will return the cached value for the key. If there is no
        value stored for the given key None will be returned.

        :key: String idenditifier for the cached value
        :returns: The cached value

        """
        return self._data.get(key)

    def all(self):
        return self._data

def setup_cache(config):
    config.add_subscriber(init_cache, NewRequest)

def init_cache(event):
    request = event.request
    settings = request.registry.settings

    if hasattr(request, "cache_item_list"):
        request.cache_item_list.clear()
    else:
        request.cache_item_list = Cache()

    if hasattr(request, "cache_item_modul"):
        request.cache_item_modul.clear()
    else:
        request.cache_item_modul = Cache()

    CACHE_TABLE_CONFIG.clear()
    if settings.get("app.cache.formconfig") != "true":
        CACHE_FORM_CONFIG.clear()
    CACHE_MODULES.clear()

CACHE_MODULES = Cache()
# GLOBAL CACHE INSTANCES
CACHE_TABLE_CONFIG = Cache()
CACHE_FORM_CONFIG = Cache()
CACHE_MISC = Cache()



import logging
import json
import sqlalchemy as sa
from datetime import datetime
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.mixins import Owned


log = logging.getLogger(__name__)


password_reset_requests = sa.Table(
    'password_reset_requests', Base.metadata,
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('uid', sa.Integer, sa.ForeignKey('users.id')),
    sa.Column('created', sa.DateTime),
    sa.Column('token', sa.String)
)

# NM-Table definitions
nm_user_roles = sa.Table(
    'nm_user_roles', Base.metadata,
    sa.Column('uid', sa.Integer, sa.ForeignKey('users.id')),
    sa.Column('rid', sa.Integer, sa.ForeignKey('roles.id'))
)

nm_user_usergroups = sa.Table(
    'nm_user_usergroups', Base.metadata,
    sa.Column('uid', sa.Integer, sa.ForeignKey('users.id')),
    sa.Column('gid', sa.Integer, sa.ForeignKey('usergroups.id'))
)

nm_action_roles = sa.Table(
    'nm_action_roles', Base.metadata,
    sa.Column('aid', sa.Integer, sa.ForeignKey('actions.id')),
    sa.Column('rid', sa.Integer, sa.ForeignKey('roles.id'))
)


class Login(Base):
    """Class to for logins of a user. Attempts to login into the
    application are stored in a table in the database. Each attemp is
    stored with the datetime, the user and the state of the login.

    This information can later be used to give information on suspicious
    logins."""
    __tablename__ = "user_logins"

    id = sa.Column(sa.Integer, primary_key=True)
    datetime = sa.Column(sa.DateTime, default=datetime.utcnow)
    success = sa.Column(sa.Boolean)
    uid = sa.Column(sa.Integer, sa.ForeignKey("users.id"))
    user = sa.orm.relationship("User", backref="logins")

    def __init__(self, user, success):
        self.success = success
        user.logins.append(self)


class PasswordResetRequest(Base):
    __tablename__ = 'password_reset_requests'
    user = sa.orm.relationship("User", backref='reset_tokens')

    def __init__(self, token):
        self.token = token
        self.created = datetime.now()

    def __str__(self):
        return self.token


class UserSetting(Base):
    __tablename__ = 'user_settings'

    id = sa.Column(sa.Integer, primary_key=True)
    settings = sa.Column(sa.Text, nullable=False, default="{}")

    def get(self, key, default):
        log.debug("Getting usersetting")
        settings = json.loads(self.settings)
        return settings.get(key, default)

    def set(self, key, value):
        settings = json.loads(self.settings)
        section = settings.get(key)
        section = value
        settings[key] = section
        dump = json.dumps(settings)
        log.debug("Setting usersetting for %s: %s" % (key, dump))
        self.settings = dump


class UserFactory(BaseFactory):

    def create(self, user, values):

        # Delete the gid which sets the default group when creating a
        # new user. The default group when creating a new user is always
        # the usergroup which gets automatically created on user
        # creation. So this value can (and must) be ignored.
        # Not removing the "gid" will otherwise cause an
        # IntegrityError while flushing the new user to the DB as the
        # usergroup with the given id is not persistent in the DB.
        if "gid" in values:
            del values["gid"]
        # Delete the sid which sets the default settings for the same
        # reasons than deleting the "gid" attribute.
        if "sid" in values:
            del values["sid"]

        new_user = BaseFactory.create(self, user, values)

        # Now create a a new Profile
        profile_factory = BaseFactory(Profile)
        profile = profile_factory.create(user, {})
        new_user.profile.append(profile)

        # Create settings
        settings_factory = BaseFactory(UserSetting)
        settings = settings_factory.create(user, {})
        new_user.settings = settings

        # A usergroup
        usergroup_factory = BaseFactory(Usergroup)
        usergroup = usergroup_factory.create(None, {})
        usergroup.name = new_user.login
        usergroup.members.append(new_user)
        # If no default group is set, then set the users group as
        # default group
        new_user.usergroup = values.get("usergroup") or usergroup
        return new_user


class User(BaseItem, Owned, Base):
    __tablename__ = 'users'
    _modul_id = 3
    _sql_eager_loads = ['roles', 'groups', 'profile', 'settings']
    id = sa.Column(sa.Integer, primary_key=True)
    login = sa.Column(sa.String, unique=True, nullable=False)
    password = sa.Column(sa.String, nullable=False)
    activated = sa.Column(sa.Boolean, default=True)
    activation_token = sa.Column(sa.String, nullable=False, default='')
    default_gid = sa.Column(sa.Integer, sa.ForeignKey('usergroups.id'))
    sid = sa.Column(sa.Integer, sa.ForeignKey('user_settings.id'))
    last_login = sa.Column(sa.DateTime)

    # Relations
    roles = sa.orm.relationship("Role",
                                secondary=nm_user_roles,
                                backref='users')
    groups = sa.orm.relationship("Usergroup",
                                 secondary=nm_user_usergroups,
                                 backref='members')
    usergroup = sa.orm.relationship("Usergroup", uselist=False,
                                    cascade="delete, all",
                                    foreign_keys=[default_gid])
    settings = sa.orm.relationship("UserSetting", uselist=False,
                                   cascade="all,delete")

    @classmethod
    def get_item_factory(cls):
        return UserFactory(cls)

    def has_role(self, role):
        """Return True if the user has the given role. Else False"
        :user: User instance
        :returns: True or False
        """
        roles = [r.name for r in self.roles]
        return role in roles

ADMIN_GROUP_ID = 1
"""Role ID your the system administration group"""
USER_GROUP_ID = 2
"""Role ID your the system user group"""
USER_ROLE_ID = 1
"""Role ID your the system user role"""


class Usergroup(BaseItem, Owned, Base):
    __tablename__ = 'usergroups'
    _modul_id = 4
    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String, unique=True, nullable=False)
    description = sa.Column(sa.String, nullable=False, default='')

    def __unicode__(self):
        return self.name


class Role(BaseItem, Owned, Base):
    """A Role is used to configure which actions are permitted to users.
    Therefor each role will have an internal list of modul actions. A
    user will be allowed to call all actions assigned to the role he is
    equiped with."""
    __tablename__ = 'roles'
    _modul_id = 5
    id = sa.Column(sa.Integer, primary_key=True)
    label = sa.Column(sa.String, unique=True, nullable=False)
    name = sa.Column(sa.String, unique=True, nullable=False)
    description = sa.Column(sa.Text, nullable=False, default='')
    admin = sa.Column(sa.Boolean, default=False)
    """Flag to set the role as administrational role which means that
    the user will gain the assigned permissions irrespective from
    checking the ownership"""

    permissions = sa.orm.relation("ActionItem",
                                  secondary=nm_action_roles,
                                  backref='roles')


class Profile(BaseItem, Owned, Base):
    __tablename__ = 'profiles'
    _modul_id = 6
    id = sa.Column(sa.Integer, primary_key=True)
    first_name = sa.Column(sa.String, nullable=True, default='')
    last_name = sa.Column(sa.String, nullable=True, default='')
    gender = sa.Column(sa.Integer)
    birthday = sa.Column(sa.Date)
    address = sa.Column(sa.Text, nullable=True, default='')
    phone = sa.Column(sa.String, nullable=True, default='')
    email = sa.Column(sa.String, nullable=True)
    web = sa.Column(sa.String, nullable=True, default='')

    # The foreign key to the user is injected from the Owned mixin.
    user = sa.orm.relation("User", cascade="all, delete",
                           backref=sa.orm.backref("profile",
                                                  cascade="all,delete-orphan"),
                           single_parent=True,
                           uselist=False)



import logging
from ringo.views.helpers import (
    get_item_form,
    render_item_form
)
from ringo.views.response import JSONResponse
from ringo.views.request import (
    handle_params,
    handle_history,
    handle_callback,
    get_item_from_request,
    get_return_value
)

log = logging.getLogger(__name__)


def read(request, callback=None, renderers=None):
    """Base method to handle read requests. Returns a dictionary of
    values used available in the rendererd template The template to
    render is defined in the view configuration.

    :request: Current request
    :callback: Current function which is called after the item has been read.
    :returns: Dictionary.
    """
    handle_history(request)
    handle_params(request)
    handle_callback(request, callback)
    rvalues = get_return_value(request)
    values = {'_roles': [str(r.name) for r in request.user.roles]}
    form = get_item_form('read', request, renderers, values=values)
    rvalues['form'] = render_item_form(request, form, values)
    return rvalues

def rest_read(request, callback=None):
    """Base method to handle read requests on the REST interface.
    Returns a JSON object of a specific item.

    :request: Current request
    :callback: Current function which is called after the item has been read.
    :returns: JSON object.
    """
    handle_callback(request, callback)
    return JSONResponse(True, get_item_from_request(request))



#!/usr/bin/env python
# encoding: utf-8
"""Modul to handle the sitetree structure of the application in order to
render a breadcrumb widget.

The sitemap is a dictionary which builds a tree hirachy of the
application. It should have the following structure::


    {
        "key": {"parent_site": $parent_key,
                "parent_item": $attritute_of_item_which_links_to_parent
                "display_item": $alternative_item_to_display,
                "display_format": "Prefix of {item}"
                "actions": {'foo': 'Foo description'}}
        ...
    }

**key**
    The *key* of the dictionary is the base *URL* of the modul.  E.g Calling
    the edit page of the user modul have the URL users/edit/1.  In this
    example *users* is the base URL. If this url is called the the entry
    point to build the breadcrumbs will be the entry in the dictionary named
    users. If no key can be found in the dictionary which named like the
    base url than no breadcrumbs can be build and no breadcrumbs are visible
    in the UI. This way you can configure breadcrumbs on only selected
    pages.

**parent_site**
    The parent_site refers to a *key* of the parent site in the sitemap.
    It must match the name of a key in the dictionary.

**parent_item**
    The item defines the name of the attribute of the currently loaded
    item (in the model) which links to the parent item in the models
    hirachy tree. Think of haven in A -> B -> C relation in your model
    and you call the page to edit C. Then item refers to the name of C's
    attribute which links to B.

**display_item**
    Usually the str representation of the related item is used as
    breadcrumb element. Setting the display_item allows to define the
    name of an attribute to an alternative value to display.

**display_format**
   Additionally to the alternative item to display you can define the *format*
   on how to display the item. This can include some addtional string to
   give some more context information.

**actions**
   You can optionally define some additional actions which should be
   displayed in the breadcrumbs. On default only the create action will
   be show as for all other actions the context is clearly defined by
   the header of the current page. If you define some custom actions for
   a modul it isn't handled by default. So if you call the custom action
   the breadcrumbs will only show the default breadcrumbs. As the last
   entry in the breadcrumbs does not provide a link you might be missing
   a way back to where you come from. To prevent this behaviour you can
   define addtional actions which will be shown.
"""

from ringo.lib.helpers.misc import get_open_url, get_item_modul

site_tree_branches = []

# Default ringo sitetree is empty. Other applications can add their part
# of a sitetree into the site_tree_branches global variable.
ringo_site_tree = {}
site_tree_branches.append(ringo_site_tree)


def url(request, item):
    if item:
        return get_open_url(request, item)
    return None


def walk_site_tree(st, el, item, request):
    """Helper method to get the elements from the sitetree.

    :st: Sitetree dictionary
    :el: Current page (key in dictionary)
    :item: currently loaded item (if loaded)
    :request: current request
    :returns: List of element used to render to breadcrumbs.

    """
    path = []
    site = st[el]

    # Get action. Currently using routes the action is always the
    # penultimate element in the URL.
    path_elements = request.path.split("/")
    if len(path_elements) > 1:
        action = site.get("actions", {}).get(path_elements[-2])
    else:
        action = None

    parent_site = site.get("parent_site")
    parent_item_attr = site.get("parent_item")

    # Check if an other item should be displayed as the item defined in the
    # item attribute. Other wise display the item.
    display_item_attr = site.get("display_item")

    if action:
        path.append((action, None))

    if display_item_attr:
        display_item = getattr(item, display_item_attr)
        display_str = site.get("display_format", "%{item}s")
        path.append((display_str.format(item=display_item),
                     url(request, item)))
    else:
        path.append((unicode(item), url(request, item)))

    if parent_site:
        parent_item = getattr(item, parent_item_attr)
        path.extend(walk_site_tree(st, parent_site, parent_item, request))
    return path


def build_breadcrumbs(request, sitetree):
    """Will return a list of breadcrumbs based on the current request
    and the given sitetree.

    :request: Current request
    :sitetree: dictionary of the sitetree
    :returns: List of breadcrumbs.
    """
    # Only build the sitemap if the modul is part of the sitemap and an
    # item is actually selected
    from ringo.views.request import get_item_from_request
    pip = request.path_info_peek()
    if pip in sitetree:
        if request.matchdict:
            item = get_item_from_request(request)
            path = walk_site_tree(sitetree, pip, item, request)
            request.session["breadcrumbs"] = path
            request.session.save()
        else:
            if request.path.find("create") > -1:
                item = get_item_from_request(request)
                modul = get_item_modul(request, item)
                path = request.session.get("breadcrumbs", [])
                return list(reversed(path)) + [(modul.get_label(), None)]
            # Reset the breadcrumbs for any other url. This means the
            # breadcrumbs are reseted every time the user calls an
            # overview page e.g. As the overview pages are usually the
            # entry point for a new working proccess the previous
            # context is lost so the breadcrumbs are.
            request.session["breadcrumbs"] = []
            return []
    else:
        request.session["breadcrumbs"] = []
        return []

    path = list(reversed(path))
    if path:
        path[-1] = (path[-1][0], None)
    return path



# -*- coding: utf-8 -*-
import os
import multiprocessing

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid',
    'SQLAlchemy',
    'alembic',
    'transaction',
    'pyramid_tm',
    'pyramid_mako',
    'pyramid_beaker',
    'pyramid_mailer',
    'zope.sqlalchemy',
    'waitress',
    'babel',
    'formbar>=0.9.4',
    'invoke',
    'dogpile.cache',
    'passlib',
    'python-dateutil',
    'fuzzy',
    'python-Levenshtein',
    'webhelpers',
    'psycopg2',
    'xlsxwriter'
]

tests_requires = [
    'pytest-ringo',
]

setup(name='ringo',
      version = '1.14.0',
      description='A simple web framework with base functionality to build web applications.',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
      ],
      author='Torsten Irländer',
      author_email='torsten@irlaender.de',
      url='https://bitbucket.org/ti/ringo',
      keywords='web wsgi bfg pylons pyramid',
      license='GNU General Public License v2 or later (GPLv2+)',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=tests_requires,
      extras_require={
          "tests": tests_requires,
          "develop": ["Sphinx", "pyramid_debugtoolbar"]
      },
      entry_points="""\
      [paste.app_factory]
      main = ringo:main
      [console_scripts]
      ringo-admin = ringo.scripts.admin:main
      [babel.extractors]
      tableconfig = ringo.lib.i18n:extract_i18n_tableconfig
      formconfig = formbar.i18n:extract_i18n_formconfig
      [pyramid.scaffold]
      ringo=ringo.scaffolds:BasicRingoTemplate
      ringo_extension=ringo.scaffolds:RingoExtensionTemplate
      """,
      message_extractors = {'ringo': [
            ('**.py', 'python', None),
            ('templates/**.html', 'mako', None),
            ('templates/**.mako', 'mako', None),
            ('views/**.xml', 'formconfig', None),
            ('views/**.json', 'tableconfig', None),
            ('static/**', 'ignore', None)]},
      )



import uuid
import logging
from ringo.model.base import BaseFactory, get_item_list
from ringo.model.user import User
from ringo.lib.table import get_table_config
from ringo.lib.helpers.misc import get_item_modul
from ringo.lib.helpers import literal
from ringo.lib.security import has_permission
from ringo.lib.renderer import (
    ListRenderer,
    DTListRenderer
)
from ringo.lib.renderer.dialogs import (
    WarningDialogRenderer
)
from ringo.views.response import JSONResponse
from ringo.views.request import (
    handle_params,
    handle_history
)

# The dictionary will hold the request handlers for bundled actions. The
# dictionary will be filled from the view definitions
_bundle_request_handlers = {}

log = logging.getLogger(__name__)


def get_bundle_action_handler(mapping, action, module):
    if module in mapping:
        views = mapping[module]
        if action in views:
            return views[action]
    return mapping["default"].get(action)


def set_bundle_action_handler(key, handler, module="default"):
    if module in _bundle_request_handlers:
        mod_actions = _bundle_request_handlers.get(module)
    else:
        _bundle_request_handlers[module] = {}
        mod_actions = _bundle_request_handlers.get(module)
    mod_actions[key] = handler
    _bundle_request_handlers[module] = mod_actions


def handle_paginating(clazz, request):
    """Returns a tupe of current page and pagesize. The default page and
    size is page on and all items on one page. This is also the default
    if pagination is not enabled for the table."""

    name = clazz.__tablename__
    # Default pagination options
    table_config = get_table_config(clazz)
    settings = request.registry.settings
    default = settings.get("layout.advanced_overviews") == "true"
    # Only set paginated if the table is the advancedsearch as the
    # simple search provides its own client sided pagination.
    if table_config.is_paginated() and table_config.is_advancedsearch(default):
        default_page = 0 # First page
        default_size = 50
    else:
        return (0, None)

    # Get pagination from session
    page = request.session.get('%s.list.pagination_page' % name, default_page)
    size = request.session.get('%s.list.pagination_size' % name, default_size)

    # Overwrite options with options from get request
    page = int(request.GET.get('pagination_page', page))
    size = request.GET.get('pagination_size', size)
    if size:
        size = int(size)
    else:
        size = None

    if 'reset' in request.params:
        request.session['%s.list.pagination_page' % name] = default_page
        request.session['%s.list.pagination_size' % name] = default_size
    else:
        request.session['%s.list.pagination_page' % name] = page
        request.session['%s.list.pagination_size' % name] = size
    request.session.save()

    return (page, size)


def handle_sorting(clazz, request):
    """Return a tuple of *fieldname* and *sortorder* (asc, desc). The
    sorting is determined in the follwoing order: First try to get the
    sorting from the current request (GET-Param). If there are no
    sorting params try to get the params saved in the session or if
    requested from a saved search. As last
    fallback use the default sorting for the table.
    """
    name = clazz.__tablename__

    # Default sorting options
    default_field = get_table_config(clazz).get_default_sort_column()
    default_order = get_table_config(clazz).get_default_sort_order()

    # Get sorting from the session. If there is no saved sorting use the
    # default value.
    field = request.session.get('%s.list.sort_field' % name, default_field)
    order = request.session.get('%s.list.sort_order' % name, default_order)

    # Get saved sorting from the the saved search.
    saved_search_id = request.params.get('saved')
    if saved_search_id:
        searches_dic = request.user.settings.get('searches', {})
        if searches_dic:
            search = searches_dic.get(name)
            if search:
                field, order = search.get(saved_search_id, [[], [], None])[1]

    # Get sorting from the request. If there is no sorting option in
    # the request then use the saved sorting options.
    field = request.GET.get('sort_field', field)
    order = request.GET.get('sort_order', order)

    # Save current sorting in the session
    if 'reset' in request.params:
        request.session['%s.list.sort_field' % name] = default_field
        request.session['%s.list.sort_order' % name] = default_order
    else:
        request.session['%s.list.sort_field' % name] = field
        request.session['%s.list.sort_order' % name] = order
    request.session.save()

    return field, order


def get_search(clazz, request):
    """Returns a list of tuples with the search word and the fieldname.
    The function will first look if there is already a saved search in
    the session for the overview of the given clazz. If there is no
    previous search the start with an empty search stack.  The following
    behavior differs depending if it is a POST or GET request:

    1. GET
    Return either an empty search stack or return the saved stack in the
    session.

    2. POST
    Get the new submitted search. If the search is not already on the
    stack, then push it.  If the search word is empty, then pop the last
    search from the stack.  Finally return the modified stack.

    Please note the this function will not save the modified search
    stack in the session! This should be done elsewhere. E.g Depending
    if the search was successfull.
    """
    name = clazz.__tablename__
    # Default search fielter
    default_search = get_table_config(clazz).get_default_search()
    # Check if there is already a saved search in the session, If not
    # use the default search.
    saved_search = (request.session.get('%s.list.search' % name, [])
                    or default_search)

    regexpr = request.session.get('%s.list.search.regexpr' % name, False)
    if "enableregexpr" in request.params:
        request.session['%s.list.search.regexpr' % name] = True
        return saved_search
    elif "disableregexpr" in request.params:
        request.session['%s.list.search.regexpr' % name] = False
        return saved_search

    if 'reset' in request.params:
        return default_search

    # If the request is not a equest from the search form then
    # abort here and return the saved search params if there are any.
    form_name = request.params.get('form')
    if form_name != "search":
        return saved_search

    saved_search_id = request.params.get('saved')
    if saved_search_id:
        searches_dic = request.user.settings.get('searches', {})
        if searches_dic:
            searches_dic_search = searches_dic.get(name)
            if searches_dic_search:
                return searches_dic_search.get(saved_search_id, [[],
                                               [], None])[0]
    elif "save" in request.params:
        return saved_search
    elif "delete" in request.params:
        return saved_search
    else:
        search = request.params.get('search')
        search_field = request.params.get('field')

    # If search is empty try to pop the last filter in the saved search
    if search == "" and len(saved_search) > 0:
        popped = saved_search.pop()
        log.debug('Popping %s from search stack' % repr(popped))

    # Iterate over the saved search. If the search is not already in the
    # stack push it.
    if search != "":
        found = False
        for x in saved_search:
            if search == x[0] and search_field == x[1]:
                found = True
                break
        if not found:
            log.debug('Adding search for "%s" in field "%s"' % (search,
                                                                search_field))
            saved_search.append((search, search_field, regexpr))
    return saved_search


def bundle_(request):
    clazz = request.context.__model__
    module = get_item_modul(request, clazz)
    handle_history(request)
    handle_params(request)
    _ = request.translate

    # Handle bundle params. If the request has the bundle_action param
    # the request is the intial request for a bundled action. In this
    # case we can delete all previous selected and stored item ids in
    # the session.
    params = request.params.mixed()
    if params.get('bundle_action'):
        request.session['%s.bundle.action' % clazz] = params.get('bundle_action')
        try:
            del request.session['%s.bundle.items' % clazz]
        except KeyError:
            pass
        request.session['%s.bundle.items' % clazz] = params.get('id', [])
    bundle_action = request.session.get('%s.bundle.action' % clazz)
    ids = request.session.get('%s.bundle.items' % clazz)

    # Check if the user selected at least one item. If not show an
    # dialog informing that the selection is empty.
    if not ids:
        title =  _("Empty selection")
        body =  _("You have not selected any item in the list. "
                  "Click 'OK' to return to the overview.")
        renderer = WarningDialogRenderer(request, title, body)
        rvalue = {}
        rvalue['dialog'] = literal(renderer.render(url=request.referrer))
        return rvalue

    # If the user only selects one single item it is not a list. So
    # convert it to a list with one item.
    if not isinstance(ids, list):
        ids = [ids]

    factory = clazz.get_item_factory()
    items = []
    ignored_items = []
    for id in ids:
        # Check if the user is allowed to call the requested action on
        # the loaded item. If so append it the the bundle, if not ignore
        # it.
        item = factory.load(id)
        if has_permission(bundle_action.lower(), item, request):
            items.append(item)
        else:
            ignored_items.append(item)

    # After checking the permissions the list of items might be empty.
    # If so show a warning to the user to inform him that the selected
    # action is not applicable.
    if not items:
        title = _("${action} not applicable",
                  mapping={"action": bundle_action})
        body = _("After checking the permissions no items remain "
                 "for which an '${action}' can be performed. "
                 "(${num} items were filtered out.)",
                 mapping={"action": bundle_action,
                          "num": len(ignored_items)})
        renderer = WarningDialogRenderer(request, title, body)
        rvalue = {}
        rvalue['dialog'] = literal(renderer.render(url=request.referrer))
        return rvalue

    handler = get_bundle_action_handler(_bundle_request_handlers,
                                        bundle_action.lower(),
                                        module.name)
    return handler(request, items, None)


def get_list_renderer(listing, request):
    """Returns the renderer for an listing.
    Allow to use DTListRenderer if the renderer configuration is set."""
    tableconfig = get_table_config(listing.clazz)
    settings = request.registry.settings
    default = settings.get("layout.advanced_overviews") == "true"
    if tableconfig.is_advancedsearch(default):
        return ListRenderer(listing)
    else:
        return DTListRenderer(listing)


def list_(request):
    clazz = request.context.__model__
    # Important! Prevent any write access on the database for this
    # request. This is needed as transform would modify the items values
    # else.
    handle_history(request)

    # If the user enters the overview page of an item we assume that the
    # user actually leaves any context where a former set backurl is
    # relevant anymore. So delete it.
    backurl = request.session.get('%s.backurl' % clazz)
    if backurl:
        # Redirect to the configured backurl.
        del request.session['%s.backurl' % clazz]
        request.session.save()

    rvalue = {}
    search = get_search(clazz, request)
    sorting = handle_sorting(clazz, request)
    pagination_page, pagination_size = handle_paginating(clazz, request)
    listing = get_item_list(request, clazz, user=request.user)
    listing.sort(sorting[0], sorting[1])
    listing.filter(search)
    listing.paginate(pagination_page, pagination_size)
    # Only save the search if there are items
    if len(listing.items) > 0:
        request.session['%s.list.search' % clazz.__tablename__] = search
        if (request.params.get('form') == "search"):
            if "save" in request.params:
                query_name = request.params.get('save')
                user = BaseFactory(User).load(request.user.id)
                searches_dic = user.settings.get('searches', {})
                searches_dic_search = searches_dic.get(clazz.__tablename__, {})

                # Check if there is already a search saved with the name
                found = False
                for xxx in searches_dic_search.values():
                    if xxx[1] == query_name:
                        found = True
                        break
                if not found:
                    searches_dic_search[str(uuid.uuid1())] = (search, sorting, query_name)
                searches_dic[clazz.__tablename__] = searches_dic_search
                user.settings.set('searches', searches_dic)
                request.db.flush()
            elif "delete" in request.params:
                query_key = request.params.get('delete')
                user = BaseFactory(User).load(request.user.id)
                searches_dic = user.settings.get('searches', {})
                searches_dic_search = searches_dic.get(clazz.__tablename__, {})
                try:
                    del searches_dic_search[query_key]
                except:
                    pass
                searches_dic[clazz.__tablename__] = searches_dic_search
                user.settings.set('searches', searches_dic)
                request.db.flush()
        request.session.save()

    renderer = get_list_renderer(listing, request)
    rendered_page = renderer.render(request)
    rvalue['clazz'] = clazz
    rvalue['listing'] = rendered_page
    rvalue['itemlist'] = listing
    return rvalue


def rest_list(request):
    """Returns a JSON objcet with all item of a clazz. The list does not
    have any capabilities for sorting or filtering

    :request: Current request.
    :returns: JSON object.

    """
    clazz = request.context.__model__
    listing = get_item_list(request, clazz, user=request.user)
    return JSONResponse(True, listing)



from ringo.lib.helpers.appinfo import (
    get_ringo_version,
    get_app_name,
    get_app_version,
    get_app_location,
    get_app_title,
    get_app_url,
    get_path_to,
    get_app_inheritance_path,
    get_app_mode,
    get_breadcrumbs
)

from webhelpers.html import literal, escape, HTML
from ringo.lib.helpers.format import (
    prettify,
    get_local_datetime,
    get_week,
    format_timedelta,
    format_datetime
)

from ringo.lib.helpers.misc import (
    serialize,
    safestring,
    age,
    get_raw_value,
    set_raw_value,
    dynamic_import,
    import_model,
    get_item_modul,
    get_modules,
    get_item_actions,
    get_action_routename,
    get_action_url,
    get_saved_searches
)



import os
import transaction
from ringo.scripts.db import get_session
from ringo.lib.helpers import get_app_location
from ringo.model.user import User
from ringo.lib.security import encrypt_password, password_generator


def handle_user_passwd_command(args):
    if args.password:
        password = args.password
    else:
        password = password_generator()
    encrypted_password = encrypt_password(password)
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)
    session = get_session(os.path.join(*path))
    try:
        user = session.query(User).filter(User.login == args.user).all()[0]
    except:
        print "User %s not found in system. You could only alter existing user's passwords" % args.user
    else:
        user.password = encrypted_password
        print "OK! Password for '%s' changed to '%s'" % (args.user, password)
    finally:
        transaction.commit()



import os
import transaction
from ringo.scripts.db import get_session
from ringo.lib.helpers import get_app_location, dynamic_import
from ringo.lib.extension import _add_modul, _load_modul_by_name
from ringo.model import extensions


def handle_app_init_command(args):
    # Create a folder for the application
    os.makedirs(args.name)
    print("Created new application folder %s" % args.name)

    # Get template for the configuration file.
    template_path = []
    template_path.append(get_app_location(args.app))
    template_path.append(args.app)
    template_path.append("scaffolds")
    template_path.append("basic")
    template_path.append("development.ini_tmpl")

    with open(os.path.join(*template_path)) as f:
        template = f.read()

    config_file = "production.ini"
    config_path = []
    config_path.append(args.name)
    config_path.append(config_file)
    with open(os.path.join(*config_path), "w") as c:
        template = template.replace("{{package}}", args.app)
        #template = template.replace("app.title = ringo", "app.title = %s"
        #                            % args.name)
        c.write(template)
    print("Created new configuration file %s in %s" % (config_file, args.name))

    print("\n")
    print("Next steps:")
    print("0. cd %s" % args.name)
    print("1. Adjust settings in your config")
    print("2. %s-admin db init --config %s" % (args.app, config_file))
    print("3. %s-admin fixtures load --config %s" % (args.app, config_file))
    print("4. Start application: pserve production.ini")


def handle_ext_init_command(args):
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)
    session = get_session(os.path.join(*path))
    for ext in extensions:
        if ext == args.name:
            if _load_modul_by_name(session, ext.replace("ringo_", "")):
                print "Extension %s already added!" % args.name
            else:
                extension = dynamic_import("%s." % args.name)
                _add_modul(extension.modul_config, None, session)
                print "Extension %s added!" % args.name
            break
    else:
        print "Extension %s not found!" % args.name


def handle_ext_delete_command(args):
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)
    session = get_session(os.path.join(*path))
    modul = _load_modul_by_name(session, args.name.replace("ringo_", ""))
    if modul:
        if args.name in extensions:
            session.delete(modul)
            transaction.commit()
            print "Extension %s deleted!" % args.name
        else:
            print "Extension %s already deleted!" % args.name
    else:
        print "Extension %s not found!" % args.name
